@@ -1,9 +1,9 @@
 from django.core.urlresolvers import reverse, NoReverseMatch
 from django.utils.safestring import mark_safe
 
-from .layout import Layout, LayoutSlice
-from .utils import render_field, flatatt, TEMPLATE_PACK
-from .exceptions import FormHelpersException
+from crispy_forms.layout import Layout, LayoutSlice
+from crispy_forms.utils import render_field, flatatt, TEMPLATE_PACK
+from crispy_forms.exceptions import FormHelpersException
 
 
 class DynamicLayoutHandler(object):
@@ -29,7 +29,8 @@ class DynamicLayoutHandler(object):
         """
         self._check_layout()
         max_level = kwargs.pop('max_level', 0)
-        filtered_layout_objects = self.layout.get_layout_objects(LayoutClasses, max_level=max_level)
+        greedy = kwargs.pop('greedy', False)
+        filtered_layout_objects = self.layout.get_layout_objects(LayoutClasses, max_level=max_level, greedy=greedy)
 
         return LayoutSlice(self.layout, filtered_layout_objects)
 
@@ -88,6 +89,18 @@ class DynamicLayoutHandler(object):
 
         return LayoutSlice(self.layout, key)
 
+    def __setitem__(self, key, value):
+        self.layout[key] = value
+
+    def __delitem__(self, key):
+        del self.layout.fields[key]
+
+    def __len__(self):
+        if self.layout is not None:
+            return len(self.layout.fields)
+        else:
+            return 0
+
 
 class FormHelper(DynamicLayoutHandler):
     """
