@@ -5,8 +5,8 @@ from django.template import Context, Template
 from django.template.loader import render_to_string
 from django.utils.html import conditional_escape
 
-from .utils import render_field, flatatt
-from .exceptions import DynamicError
+from crispy_forms.utils import render_field, flatatt
+from crispy_forms.exceptions import DynamicError
 
 TEMPLATE_PACK = getattr(settings, 'CRISPY_TEMPLATE_PACK', 'bootstrap')
 
@@ -18,12 +18,19 @@ class LayoutObject(object):
     def __setitem__(self, slice, value):
         self.fields[slice] = value
 
+    def __delitem__(self, slice):
+        del self.fields[slice]
+
+    def __len__(self):
+        return len(self.fields)
+
     def __getattr__(self, name):
         """
         This allows us to access self.fields list methods like append or insert, without
         having to declaee them one by one
         """
-        if hasattr(self.fields, name):
+        # Check necessary for unpickling, see #107
+        if 'fields' in self.__dict__ and hasattr(self.fields, name):
             return getattr(self.fields, name)
         else:
             return object.__getattribute__(self, name)
@@ -151,51 +158,66 @@ class LayoutSlice(object):
             else:
                 return LayoutClass(fields, **kwargs)
 
-    def wrap(self, LayoutClass, *args, **kwargs):
+    def pre_map(self, function):
         """
-        Wraps each pointer in `self.slice` under a `LayoutClass` instance with
-        `args` and `kwargs` passed.
+        Iterates over layout objects pointed in `self.slice` executing `function` on them.
+        It passes `function` penultimate layout object and the position where to find last one
         """
         if isinstance(self.slice, slice):
             for i in range(*self.slice.indices(len(self.layout.fields))):
-                self.layout.fields[i] = self.wrapped_object(LayoutClass, self.layout.fields[i], *args, **kwargs)
+                function(self.layout, i)
 
         elif isinstance(self.slice, list):
             # A list of pointers  Ex: [[[0, 0], 'div'], [[0, 2, 3], 'field_name']]
             for pointer in self.slice:
                 position = pointer[0]
 
-                # If it's pointing first level, there is no need to traverse
+                # If it's pointing first level
                 if len(position) == 1:
-                    self.layout.fields[position[-1]] = self.wrapped_object(
-                        LayoutClass, self.layout.fields[position[-1]], *args, **kwargs
-                    )
+                    function(self.layout, position[-1])
                 else:
                     layout_object = self.layout.fields[position[0]]
                     for i in position[1:-1]:
                         layout_object = layout_object.fields[i]
 
                     try:
-                        # If layout object has a fields attribute
-                        if hasattr(layout_object, 'fields'):
-                            layout_object.fields[position[-1]] = self.wrapped_object(
-                                LayoutClass, layout_object.fields[position[-1]], *args, **kwargs
-                            )
-                        # Otherwise it's a basestring (a field name)
-                        else:
-                            self.layout.fields[position[0]] = self.wrapped_object(
-                                LayoutClass, layout_object, *args, **kwargs
-                            )
+                        function(layout_object, position[-1])
                     except IndexError:
                         # We could avoid this exception, recalculating pointers.
                         # However this case is most of the time an undesired behavior
                         raise DynamicError("Trying to wrap a field within an already wrapped field, \
                             recheck your filter or layout")
 
+    def wrap(self, LayoutClass, *args, **kwargs):
+        """
+        Wraps every layout object pointed in `self.slice` under a `LayoutClass` instance with
+        `args` and `kwargs` passed.
+        """
+        def wrap_object(layout_object, j):
+            layout_object.fields[j] = self.wrapped_object(
+                LayoutClass, layout_object.fields[j], *args, **kwargs
+            )
+
+        self.pre_map(wrap_object)
+
+    def wrap_once(self, LayoutClass, *args, **kwargs):
+        """
+        Wraps every layout object pointed in `self.slice` under a `LayoutClass` instance with
+        `args` and `kwargs` passed, unless layout object's parent is already a subclass of
+        `LayoutClass`.
+        """
+        def wrap_object_once(layout_object, j):
+            if not isinstance(layout_object, LayoutClass):
+                layout_object.fields[j] = self.wrapped_object(
+                    LayoutClass, layout_object.fields[j], *args, **kwargs
+                )
+
+        self.pre_map(wrap_object_once)
+
     def wrap_together(self, LayoutClass, *args, **kwargs):
         """
-        Wraps pointers in `self.slice` together under a `LayoutClass` instance with
-        `args` and `kwargs` passed.
+        Wraps all layout objects pointed in `self.slice` together under a `LayoutClass`
+        instance with `args` and `kwargs` passed.
         """
         if isinstance(self.slice, slice):
             # The start of the slice is replaced
@@ -211,6 +233,36 @@ class LayoutSlice(object):
         elif isinstance(self.slice, list):
             raise DynamicError("wrap_together doesn't work with filter, only with [] operator")
 
+    def map(self, function):
+        """
+        Iterates over layout objects pointed in `self.slice` executing `function` on them
+        It passes `function` last layout object
+        """
+        if isinstance(self.slice, slice):
+            for i in range(*self.slice.indices(len(self.layout.fields))):
+                function(self.layout.fields[i])
+
+        elif isinstance(self.slice, list):
+            # A list of pointers  Ex: [[[0, 0], 'div'], [[0, 2, 3], 'field_name']]
+            for pointer in self.slice:
+                position = pointer[0]
+
+                layout_object = self.layout.fields[position[0]]
+                for i in position[1:]:
+                    layout_object = layout_object.fields[i]
+
+                function(layout_object)
+
+    def update_attributes(self, **kwargs):
+        """
+        Updates attributes of every layout object pointed in `self.slice` using kwargs
+        """
+        def update_attrs(layout_object):
+            if hasattr(layout_object, 'attrs'):
+                layout_object.attrs.update(kwargs)
+
+        self.map(update_attrs)
+
 
 class ButtonHolder(LayoutObject):
     """
@@ -255,7 +307,7 @@ class BaseInput(object):
         self.id = kwargs.get('css_id', '')
         self.attrs = {}
 
-        if kwargs.has_key('css_class'):
+        if 'css_class' in kwargs:
             self.field_classes += ' %s' % kwargs.pop('css_class')
 
         self.template = kwargs.pop('template', self.template)
@@ -380,9 +432,11 @@ class MultiField(LayoutObject):
 
         fields_output = u''
         for field in self.fields:
-            fields_output += render_field(field, form, form_style, context,
+            fields_output += render_field(
+                field, form, form_style, context,
                 self.field_template, self.label_class, layout_object=self,
-                template_pack=template_pack)
+                template_pack=template_pack
+            )
 
         context.update({'multifield': self, 'fields_output': fields_output})
         return render_to_string(self.template, context)
@@ -401,7 +455,7 @@ class Div(LayoutObject):
     def __init__(self, *fields, **kwargs):
         self.fields = list(fields)
 
-        if hasattr(self, 'css_class') and kwargs.has_key('css_class'):
+        if hasattr(self, 'css_class') and 'css_class' in kwargs:
             self.css_class += ' %s' % kwargs.pop('css_class')
         if not hasattr(self, 'css_class'):
             self.css_class = kwargs.pop('css_class', None)
@@ -471,18 +525,22 @@ class Field(LayoutObject):
         if not hasattr(self, 'attrs'):
             self.attrs = {}
 
-        if kwargs.has_key('css_class'):
+        if 'css_class' in kwargs:
             if 'class' in self.attrs:
                 self.attrs['class'] += " %s" % kwargs.pop('css_class')
             else:
                 self.attrs['class'] = kwargs.pop('css_class')
 
+        self.wrapper_class = kwargs.pop('wrapper_class', None)
         self.template = kwargs.pop('template', self.template)
 
         # We use kwargs as HTML attributes, turning data_id='test' into data-id='test'
-        self.attrs.update(dict([(k.replace('_', '-'), conditional_escape(v)) for k,v in kwargs.items()]))
+        self.attrs.update(dict([(k.replace('_', '-'), conditional_escape(v)) for k, v in kwargs.items()]))
 
     def render(self, form, form_style, context, template_pack=TEMPLATE_PACK):
+        if hasattr(self, 'wrapper_class'):
+            context['wrapper_class'] = self.wrapper_class
+
         html = ''
         for field in self.fields:
             html += render_field(field, form, form_style, context, template=self.template, attrs=self.attrs, template_pack=template_pack)
