@@ -1,13 +1,17 @@
 # -*- coding: utf-8 -*-
 from django.conf import settings
+from django.forms import forms
 from django.forms.formsets import BaseFormSet
 from django.template import Context
 from django.template.loader import get_template
 from django.utils.functional import memoize
 from django import template
 
+from crispy_forms.exceptions import CrispyError
 
 TEMPLATE_PACK = getattr(settings, 'CRISPY_TEMPLATE_PACK', 'bootstrap')
+
+
 def uni_formset_template(template_pack=TEMPLATE_PACK):
     return get_template('%s/uni_formset.html' % template_pack)
 uni_formset_template = memoize(uni_formset_template, {}, 1)
@@ -19,7 +23,7 @@ uni_form_template = memoize(uni_form_template, {}, 1)
 
 register = template.Library()
 
-@register.filter(name='as_uni_form')
+
 @register.filter(name='crispy')
 def as_crispy_form(form, template_pack=TEMPLATE_PACK):
     """
@@ -43,7 +47,7 @@ def as_crispy_form(form, template_pack=TEMPLATE_PACK):
         c = Context({'form': form, 'form_show_errors': True})
     return template.render(c)
 
-@register.filter(name='as_uni_errors')
+
 @register.filter(name='as_crispy_errors')
 def as_crispy_errors(form, template_pack=TEMPLATE_PACK):
     """
@@ -59,11 +63,10 @@ def as_crispy_errors(form, template_pack=TEMPLATE_PACK):
         c = Context({'formset': form})
     else:
         template = get_template('%s/errors.html' % template_pack)
-        c = Context({'form':form})
+        c = Context({'form': form})
     return template.render(c)
 
 
-@register.filter(name='as_uni_field')
 @register.filter(name='as_crispy_field')
 def as_crispy_field(field, template_pack=TEMPLATE_PACK):
     """
@@ -74,6 +77,9 @@ def as_crispy_field(field, template_pack=TEMPLATE_PACK):
         or
         {{ form.field|as_crispy_field:"bootstrap" }}
     """
+    if not isinstance(field, forms.BoundField):
+        raise CrispyError('|as_crispy_field got passed an invalid or inexistent field')
+
     template = get_template('%s/field.html' % template_pack)
-    c = Context({'field':field, 'form_show_errors': True})
+    c = Context({'field': field, 'form_show_errors': True})
     return template.render(c)
