@@ -5,6 +5,7 @@ import django
 from django import forms
 from django.conf import settings
 from django.core.urlresolvers import reverse
+from django.forms.forms import BoundField
 from django.forms.models import formset_factory
 from django.template import Context, TemplateSyntaxError
 from django.template.loader import get_template_from_string
@@ -21,7 +22,8 @@ from crispy_forms.layout import (
 )
 from crispy_forms.bootstrap import (
     AppendedPrependedText, AppendedText, PrependedText, InlineCheckboxes,
-    FieldWithButtons, StrictButton, InlineRadios, Tab, TabHolder
+    FieldWithButtons, StrictButton, InlineRadios, Tab, TabHolder,
+    AccordionGroup, Accordion
 )
 from crispy_forms.utils import render_crispy_form
 from crispy_forms.templatetags.crispy_forms_tags import CrispyFormNode
@@ -106,6 +108,20 @@ class TestBasicFunctionalityTags(TestCase):
         html = template.render(c)
         self.assertTrue('email-fields' in html)
 
+    def test_crispy_field_and_class_converters(self):
+        template = get_template_from_string(u"""
+            {% load crispy_forms_field %}
+            {% crispy_field testField 'class' 'error' %}
+        """)
+        test_form = TestForm()
+        field_instance = test_form.fields['email']
+        bound_field = BoundField(test_form, field_instance, 'email')
+
+        c = Context({'testField': bound_field})
+        html = template.render(c)
+        self.assertTrue('error' in html)
+        self.assertTrue('inputtext' in html)
+
 
 class TestFormHelpers(TestCase):
     urls = 'crispy_forms.tests.urls'
@@ -594,7 +610,7 @@ class TestFormLayout(TestCase):
         c = Context({'form': form, 'form2': form2})
         html = template.render(c)
 
-        self.assertEqual(html.count('<input type="text" name="comment"'), 2)
+        self.assertEqual(html.count('name="comment"'), 2)
         self.assertEqual(html.count('name="is_company"'), 1)
 
     def test_hidden_fields(self):
@@ -990,7 +1006,7 @@ class TestLayoutObjects(TestCase):
         test_form = TestForm()
         test_form.helper = FormHelper()
         test_form.helper.layout = Layout(
-            Field('email', type="hidden", data_mierda=12),
+            Field('email', type="hidden", data_test=12),
             Field('datetime_field'),
         )
 
@@ -1000,10 +1016,16 @@ class TestLayoutObjects(TestCase):
         html = template.render(c)
 
         # Check form parameters
-        self.assertEqual(html.count('<input type="hidden" data-mierda="12" name="email"'), 1)
+        self.assertEqual(html.count('data-test="12"'), 1)
+        self.assertEqual(html.count('name="email"'), 1)
         self.assertEqual(html.count('class="dateinput"'), 1)
         self.assertEqual(html.count('class="timeinput"'), 1)
 
+    def test_field_wrapper_class(self):
+        html = Field('email', wrapper_class="testing").render(TestForm(), "", Context())
+        if settings.CRISPY_TEMPLATE_PACK == 'bootstrap':
+            self.assertEqual(html.count('class="control-group testing"'), 1)
+
     def test_appended_prepended_text(self):
         template = get_template_from_string(u"""
             {% load crispy_forms_tags %}
@@ -1037,6 +1059,32 @@ class TestLayoutObjects(TestCase):
 
         self.assertEqual(html.count('radio inline"'), 2)
 
+    def test_accordion_and_accordiongroup(self):
+        test_form = TestForm()
+        test_form.helper = FormHelper()
+        test_form.helper.layout = Layout(
+            Accordion(
+                AccordionGroup(
+                    'one',
+                    'first_name'
+                ),
+                AccordionGroup(
+                    'two',
+                    'password1',
+                    'password2'
+                )
+            )
+        )
+        html = render_crispy_form(test_form)
+
+        self.assertEqual(html.count('<div class="accordion"'), 1)
+        self.assertEqual(html.count('<div class="accordion-group">'), 2)
+        self.assertEqual(html.count('<div id="one"'), 1)
+        self.assertEqual(html.count('<div id="two"'), 1)
+        self.assertEqual(html.count('name="first_name"'), 1)
+        self.assertEqual(html.count('name="password1"'), 1)
+        self.assertEqual(html.count('name="password2"'), 1)
+
     def test_tab_and_tabholder(self):
         test_form = TestForm()
         test_form.helper = FormHelper()
@@ -1189,6 +1237,46 @@ class TestDynamicLayouts(TestCase):
         self.assertEqual(layout.fields[1], 'password1')
         self.assertEqual(layout.fields[2], 'password2')
 
+    def test_update_attributes_and_wrap_once(self):
+        helper = FormHelper()
+        layout = Layout(
+            'email',
+            Field('password1'),
+            'password2',
+        )
+        helper.layout = layout
+        helper.filter(Field).update_attributes(readonly=True)
+        self.assertTrue(isinstance(layout[1], Field))
+        self.assertEqual(layout[1].attrs, {'readonly': True})
+
+        layout = Layout(
+            'email',
+            Div(Field('password1')),
+            'password2',
+        )
+        helper.layout = layout
+        helper.filter(Field, max_level=2).update_attributes(readonly=True)
+        self.assertTrue(isinstance(layout[1][0], Field))
+        self.assertEqual(layout[1][0].attrs, {'readonly': True})
+
+        layout = Layout(
+            'email',
+            Div(Field('password1')),
+            'password2',
+        )
+        helper.layout = layout
+
+        helper.filter(basestring, greedy=True).wrap_once(Field)
+        helper.filter(Field, greedy=True).update_attributes(readonly=True)
+
+        self.assertTrue(isinstance(layout[0], Field))
+        self.assertTrue(isinstance(layout[1][0], Field))
+        self.assertTrue(isinstance(layout[1][0][0], basestring))
+        self.assertTrue(isinstance(layout[2], Field))
+        self.assertEqual(layout[1][0].attrs, {'readonly': True})
+        self.assertEqual(layout[0].attrs, {'readonly': True})
+        self.assertEqual(layout[2].attrs, {'readonly': True})
+
     def test_get_layout_objects(self):
         layout_1 = Layout(
             Div()
@@ -1437,18 +1525,45 @@ class TestDynamicLayouts(TestCase):
         form.helper.layout = self.advanced_layout
         self.assertRaises(FormHelpersException, lambda: form.helper.filter_by_widget(forms.PasswordInput))
 
-    def test_getitem_by_field_name(self):
-        form = TestForm()
-        form.helper = FormHelper(form)
+    def test_formhelper__getitem__(self):
+        helper = FormHelper()
         layout = Layout(
             Div('email'),
             'password1',
         )
-        form.helper.layout = layout
-        form.helper['email'].wrap(Field, css_class='hero')
-        self.assertTrue(isinstance(layout.fields[0].fields[0], Field))
+        helper.layout = layout
+        helper['email'].wrap(Field, css_class='hero')
+        self.assertTrue(isinstance(layout[0][0], Field))
 
-    def test_getitem_layout_object(self):
+    def test_formhelper__setitem__(self):
+        helper = FormHelper()
+        layout = Layout(
+            'first_field',
+            Div('email')
+        )
+        helper.layout = layout
+        helper[0] = 'replaced'
+        self.assertEqual(layout[0], 'replaced')
+
+    def test_formhelper__delitem__and__len__(self):
+        helper = FormHelper()
+        layout = Layout(
+            'first_field',
+            Div('email')
+        )
+        helper.layout = layout
+        del helper[0]
+        self.assertEqual(len(helper), 1)
+
+    def test__delitem__and__len__layout_object(self):
+        layout = Layout(
+            'first_field',
+            Div('email')
+        )
+        del layout[0]
+        self.assertEqual(len(layout), 1)
+
+    def test__getitem__layout_object(self):
         layout = Layout(
             Div(
                 Div(
@@ -1465,7 +1580,7 @@ class TestDynamicLayouts(TestCase):
         self.assertTrue(isinstance(layout[0][1][0], basestring))
         self.assertTrue(isinstance(layout[0][2], basestring))
 
-    def test_append_layout_object(self):
+    def test__getattr__append_layout_object(self):
         layout = Layout(
             Div('email')
         )
@@ -1474,7 +1589,7 @@ class TestDynamicLayouts(TestCase):
         self.assertTrue(isinstance(layout[0][0], basestring))
         self.assertTrue(isinstance(layout[1], basestring))
 
-    def test_setitem_layout_object(self):
+    def test__setitem__layout_object(self):
         layout = Layout(
             Div('email')
         )
